SystemOrganization addCategory: #GODSpreadsheet!

GODData subclass: #SSSpreadsheetData
	instanceVariableNames: 'sheets'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODSpreadsheet'!

!SSSpreadsheetData commentStamp: 'jjs 10/18/2014 17:56' prior: 0!
Creates a spreadsheet with a set of sheets!

!SSSpreadsheetData class methodsFor: 'initializing' stamp: 'foa 10/18/2014 16:13'!
fromCSV: filePath
	"initialize a SSSpreadsheetData from specified CSV"
	|file line row columns ssdata sheet count|
	ssdata := SSSpreadsheetData new.
	sheet := ssdata createSheetWithName: 'default'.
	
	file := FileStream fileNamed: filePath.
	[file atEnd] whileFalse:
		[line := file nextLine.
		columns := line findTokens: ',' escapedBy: '"'.
		row := sheet createRow.
		count := 1.
		columns do: [:item | row createCellAtColumnIndex: count withValue: item. count := count + 1]
		].
	^ssdata! !

!SSSpreadsheetData class methodsFor: 'initializing' stamp: 'dhk 10/17/2014 19:21'!
fromFile: filePath
	"initialize a SSSpreadsheetData from specified path"
	|extension|
	extension := (filePath subStrings: '.') last asUppercase.
	extension = 'CSV'
		ifTrue: [^self fromCSV: filePath].
	extension = 'ODS'
		ifTrue: [^self fromODS: filePath].! !

!SSSpreadsheetData class methodsFor: 'initializing' stamp: 'foa 10/18/2014 18:16'!
fromODS: filePath
	"initialize a SSSpreadsheetData from specified ODS"
	
	| odsFile contentsXML data root ssdata sheet row colIndex rowsRepeat columnsRepeat |
	
	odsFile := ZipArchive new readFrom: filePath.
	contentsXML := odsFile member: 'content.xml'.
	data := XMLDOMParser parseDocumentFrom: contentsXML contents readStream.
	root := data elementAt: #'office:document-content'.
	
	ssdata := SSSpreadsheetData new.
	root tagsNamed: #'table:table' do: [ :table |
		sheet := ssdata createSheetWithName: (table attributeAt: 'table:name').
		table tagsNamed: #'table:table-row' do: [ :trow |
			rowsRepeat := trow attributeAt: #'table:number-rows-repeated'.
			rowsRepeat isNil ifTrue: [ rowsRepeat := 1 ] ifFalse: [ rowsRepeat := rowsRepeat asNumber ].
			rowsRepeat timesRepeat: [
				row := sheet createRow.
				colIndex := 1.
				trow tagsNamed: #'table:table-cell' do: [ :cell |
					columnsRepeat := cell attributeAt: #'table:number-columns-repeated'.
					columnsRepeat isNil ifTrue: [ columnsRepeat := 1 ] ifFalse: [ columnsRepeat := columnsRepeat asNumber ].
					columnsRepeat timesRepeat: [
						row createCellAtColumnIndex: colIndex withValue: (SSSpreadsheetData getODSCellValue: cell).
						colIndex := colIndex + 1.
						].
					].
				].
			].
		].
	^ssdata.! !

!SSSpreadsheetData class methodsFor: 'utility' stamp: 'foa 10/18/2014 17:01'!
getODSCellValue: aCell
	"Gets the value of the given cell (from a ODS spreadsheet"

	| internalData |
	internalData := aCell firstTagNamed: #'text:p'.
	internalData isNil ifTrue: [ ^'' ].
	
	^internalData elementsAndContents first.! !

!SSSpreadsheetData methodsFor: 'adding' stamp: 'dhk 10/29/2014 22:16'!
createSheetWithName: name
	"creates a sheet with name 'name'"
	|sheet|
	sheet := SSSheet new: name.
	sheet parent: self.
	sheets add: sheet.
	sheet index: sheets size.
	^sheet! !

!SSSpreadsheetData methodsFor: 'accessing' stamp: 'dhk 10/17/2014 18:17'!
getSheet: sheetNumber
	"returns the sheet with the sheetNumber index"
	
	^sheets at: sheetNumber.! !

!SSSpreadsheetData methodsFor: 'accessing' stamp: 'dhk 10/17/2014 18:23'!
getSheetWithName: sheetName
	"returns the sheet with name sheetName"
	sheets do:
		[:sheet | (sheet name = sheetName) ifTrue: [^sheet]].
	^nil! !

!SSSpreadsheetData methodsFor: 'initialize' stamp: 'jjs 10/18/2014 17:55'!
initialize
	"creates a new empty set of sheets and returns the set of sheets"
	sheets := OrderedCollection new.
	^self! !

!SSSpreadsheetData methodsFor: 'accessing' stamp: 'dhk 9/27/2014 17:23'!
numberOfSheets
	"returns how many sheets exists in this GODSpreadsheetData"

	^sheets size! !

!SSSpreadsheetData methodsFor: 'accessing' stamp: 'jjs 10/18/2014 17:39'!
renameSheet: sheetName toName: aName
	"renames a sheet with name sheetName to the name aName"
	|sheet|
	sheet := (self getSheetWithName: sheetName).
	sheet changeName: aName.! !

!SSSpreadsheetData methodsFor: 'file out' stamp: 'dhk 10/2/2014 21:51'!
toFile: filePath
	"save this SSSpreadsheetData to specified path. Returns false if there is an error"

	^true! !

Object subclass: #SSCell
	instanceVariableNames: 'value parent column'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODSpreadsheet'!

!SSCell commentStamp: 'jjs 10/18/2014 17:46' prior: 0!
creates an cell object with a parent row, column index and cell's value!

!SSCell class methodsFor: 'initialize' stamp: 'dhk 10/17/2014 16:45'!
newWithParent: aParent atColumn: aColumnName

	^((self new) parent: aParent) column: aColumnName.! !

!SSCell methodsFor: 'accessing' stamp: 'dhk 10/26/2014 15:44'!
column
	"Return the column name of this cell.
	We expect column names as used in common office spreadsheet editing applications. Columns start from A to Z, then AA to AZ, BA and so on..."

	^column! !

!SSCell methodsFor: 'accessing' stamp: 'dhk 10/29/2014 17:51'!
column: aColumn
	"Set the column name of this cell.
	We expect column names as used in common office spreadsheet editing applications. Columns start from A to Z, then AA to AZ, BA and so on..."

	column := (aColumn asUppercase).! !

!SSCell methodsFor: 'accessing' stamp: 'dhk 10/17/2014 16:22'!
getColumnIndex
	"Returns the column index of this cell."

	^SSSheet columnNameToIndex: column! !

!SSCell methodsFor: 'accessing' stamp: 'dhk 10/17/2014 16:47'!
getParentRow
	"Returns the Row which this cell belongs to."

	^self parent.! !

!SSCell methodsFor: 'accessing' stamp: 'dhk 10/17/2014 16:23'!
getRowIndex
	"Returns the row index of this cell. "

	^parent index.! !

!SSCell methodsFor: 'accessing' stamp: 'dhk 10/17/2014 16:47'!
parent
	"Returns the Row which this cell belongs to."

	^parent.! !

!SSCell methodsFor: 'accessing' stamp: 'dhk 10/17/2014 16:06'!
parent: aParent
	"set the parent of this cell"

	parent := aParent! !

!SSCell methodsFor: 'accessing' stamp: 'dhk 10/2/2014 21:40'!
value
	"returns the value of this cell"

	^value! !

!SSCell methodsFor: 'accessing' stamp: 'dhk 10/2/2014 21:40'!
value: aValue
	"set the value of this cell"

	value := aValue! !

Object subclass: #SSRow
	instanceVariableNames: 'cells parent index'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODSpreadsheet'!

!SSRow commentStamp: 'jjs 10/18/2014 17:49' prior: 0!
Creates and row object with a set of cells, parent sheet and row index!

!SSRow methodsFor: 'adding' stamp: 'dhk 10/26/2014 15:46'!
addCell: aCell atColumn: columnName
	"add aCell at the column with name columnName
	We expect column names as used in common office spreadsheet editing applications. Columns start from A to Z, then AA to AZ, BA and so on..."
	self addCell: aCell atColumnIndex: (SSSheet columnNameToIndex: columnName).
	
	
	
	! !

!SSRow methodsFor: 'adding' stamp: 'dhk 10/26/2014 15:46'!
addCell: aCell atColumnIndex: columnIndex
	"add a cell in this row at the specified column index (an integer)"
	[((cells size)) < columnIndex]
		whileTrue:
		[cells add: (SSCell newWithParent: self atColumn: (SSSheet columnIndexToName: cells size + 1))].
	cells at: columnIndex put: aCell.! !

!SSRow methodsFor: 'adding' stamp: 'dhk 10/26/2014 15:46'!
createCellAtColumn: columnName withValue: aValue
	"create cell at the column with name columnName and value aValue
	We expect column names as used in common office spreadsheet editing applications. Columns start from A to Z, then AA to AZ, BA and so on..."
	|aCell|
	aCell := (SSCell newWithParent: self atColumn: columnName) value: aValue.
	self addCell: aCell atColumn: columnName.
	^aCell
	! !

!SSRow methodsFor: 'adding' stamp: 'dhk 10/26/2014 15:46'!
createCellAtColumnIndex: columnIndex withValue: aValue
	"create cell at the column index columnIndex (an integer) with value aValue"
	^self createCellAtColumn: (SSSheet columnIndexToName: columnIndex) withValue: aValue
	
	! !

!SSRow methodsFor: 'deleting' stamp: 'dhk 10/26/2014 15:46'!
deleteCell: columnName
	"delete the cell at the column with name columnName
	We expect column names as used in common office spreadsheet editing applications. Columns start from A to Z, then AA to AZ, BA and so on..."
	self deleteCellAtIndex: (SSSheet columnNameToIndex: columnName).! !

!SSRow methodsFor: 'deleting' stamp: 'dhk 10/26/2014 15:46'!
deleteCellAtIndex: columnIndex
	"delete the cell at specified column index (an integer)"
	cells removeAt: columnIndex.! !

!SSRow methodsFor: 'accessing' stamp: 'dhk 10/26/2014 15:45'!
getCell: columnName
	"get the cell in this row at the column with name columnName
	We expect column names as used in common office spreadsheet editing applications. Columns start from A to Z, then AA to AZ, BA and so on..."

	^self getCellAtIndex: (SSSheet columnNameToIndex: columnName).! !

!SSRow methodsFor: 'accessing' stamp: 'dhk 10/26/2014 15:45'!
getCellAtIndex: columnIndex
	"get the cell in this row at columnIndex (an integer)"

	^cells at: columnIndex.! !

!SSRow methodsFor: 'accessing' stamp: 'dhk 10/26/2014 15:45'!
index
	"Returns the index (an integer) of this row in the parent sheet."

	^index.! !

!SSRow methodsFor: 'accessing' stamp: 'dhk 10/17/2014 16:16'!
index: aIndex
	"set the index of this cell"
	index := aIndex! !

!SSRow methodsFor: 'initialize' stamp: 'jjs 10/18/2014 17:28'!
initialize
      "initialize this instance with an empty cell collection"
	cells := OrderedCollection new.! !

!SSRow methodsFor: 'accessing' stamp: 'dhk 10/17/2014 16:52'!
parent
	"Returns the sheet which this row belongs to."

	^parent.! !

!SSRow methodsFor: 'accessing' stamp: 'jjs 10/18/2014 17:28'!
parent: aParent
     "Sets the sheet which this row belongs to."

	parent := aParent.! !

Object subclass: #SSSheet
	instanceVariableNames: 'sheetName rows parent index'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODSpreadsheet'!

!SSSheet commentStamp: 'jjs 10/18/2014 17:51' prior: 0!
Creates an spreadsheet object with a spreadsheetData parent, a sheet name, a set of rows and a sheet index !

!SSSheet class methodsFor: 'utility' stamp: 'foa 10/17/2014 17:51'!
columnIndexToName: aIndex
	"Utility method to convert a column index value to a column name."
	
	| rest subIndex |
	aIndex <= 26
		ifTrue:	[^(aIndex+64) asCharacter asString].
	rest := (aIndex - 1) \\ 26.
	subIndex := (aIndex - 1) // 26.
	^(self columnIndexToName: subIndex), ((rest + 65) asCharacter asString)! !

!SSSheet class methodsFor: 'utility' stamp: 'dhk 10/29/2014 17:50'!
columnNameToIndex: columnName
	"Utility method to convert a column name value to a column index."
     "index = 0
	l = list(name)
	index = ord(l[0]) - 64
	for i in xrange(1, len(l)):
		index *= 26
		index += ord(l[i]) - 64
	return index"
	
	| index aName | 
	aName := (columnName asUppercase).
	index := (aName first asInteger) - 64.
	aName allButFirstDo: [ :char |
		index := (index * 26) + ((char asInteger) - 64)].
	^index! !

!SSSheet class methodsFor: 'instance creation' stamp: 'dhk 9/27/2014 16:35'!
new: aName
	^(self new) changeName: aName! !

!SSSheet methodsFor: 'private' stamp: 'dhk 9/27/2014 16:39'!
changeName: aName
	"change the name of this sheet"

	sheetName := aName! !

!SSSheet methodsFor: 'accessing' stamp: 'dhk 10/29/2014 18:05'!
createCellAtRow: rowIndex atColumn: columnName withValue: aValue
	"creates a cell at this specified address with value aValue and returns it
	We expect column names as used in common office spreadsheet editing applications. Columns start from A to Z, then AA to AZ, BA and so on..."
	| row cell |
	rows size < rowIndex
		ifTrue: [row := (self createRowAtIndex: rowIndex)]
		ifFalse:[row := rows at: rowIndex].
	cell := row createCellAtColumn: columnName withValue: aValue.
	^cell! !

!SSSheet methodsFor: 'accessing' stamp: 'dhk 10/29/2014 18:07'!
createCellAtRow: rowIndex atColumnIndex: columnIndex withValue: aValue
	"creates a cell at this specified address with value aValue and returns it"
	^self createCellAtRow: rowIndex atColumn: (SSSheet columnIndexToName: columnIndex) withValue: aValue.! !

!SSSheet methodsFor: 'adding' stamp: 'jjs 10/18/2014 17:32'!
createRow
	"creates a new empty row after the last row in this sheet and returns it"
	|row|
	row := ((SSRow new) parent: self).
	rows add: row.
	row index: (rows size).
	^row.! !

!SSSheet methodsFor: 'adding' stamp: 'dhk 10/29/2014 22:10'!
createRowAtIndex: aIndex
	"creates a new empty row at a specified index in this sheet.
	This method creates empty rows if needed"
	
	[((rows size) + 1) < aIndex]
		whileTrue:
			[self createRow].
	^self createRow! !

!SSSheet methodsFor: 'deleting' stamp: 'dhk 10/17/2014 19:28'!
deleteRow: aIndex
	"delete row at index aIndex"
	
	rows removeAt: aIndex.! !

!SSSheet methodsFor: 'accessing' stamp: 'dhk 10/26/2014 15:48'!
getCellAtRow: rowIndex atColumn: columnName
	"returns the cell at this specified address
	We expect column names as used in common office spreadsheet editing applications. Columns start from A to Z, then AA to AZ, BA and so on..."

	^self getCellAtRow: rowIndex atColumnIndex: (SSSheet columnNameToIndex: columnName).! !

!SSSheet methodsFor: 'accessing' stamp: 'dhk 10/17/2014 17:22'!
getCellAtRow: rowIndex atColumnIndex: columnIndex
	"returns the cell at this specified address"

	^(rows at: rowIndex) getCellAtIndex: columnIndex.! !

!SSSheet methodsFor: 'accessing' stamp: 'dhk 10/17/2014 17:20'!
getRow: rowIndex
	"returns the row with the specified index"

	^rows at: rowIndex! !

!SSSheet methodsFor: 'accessing' stamp: 'dhk 10/29/2014 22:16'!
index
	"return the index of this sheet in the SpreadsheetData"

	^index.! !

!SSSheet methodsFor: 'accessing' stamp: 'dhk 10/29/2014 22:16'!
index: aIndex
	"set the index of this sheet in the SpreadsheetData"

	index := aIndex.! !

!SSSheet methodsFor: 'initialize' stamp: 'dhk 10/17/2014 17:18'!
initialize
	rows := OrderedCollection new.! !

!SSSheet methodsFor: 'accessing' stamp: 'dhk 10/2/2014 21:39'!
name
	"returns the name of this sheet"

	^sheetName! !

!SSSheet methodsFor: 'accessing' stamp: 'dhk 10/17/2014 17:19'!
parent
	"return the SSSpreadsheetData which this sheet belongs to"

	^parent! !

!SSSheet methodsFor: 'accessing' stamp: 'dhk 10/17/2014 17:19'!
parent: aParent
	"set the SSSpreadsheetData which this sheet belongs to"

	parent := aParent! !
